<?xml version="1.0"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/TR/WD-xsl">

  <xsl:script><![CDATA[
    function sectionNum(e) {
      if (e)
      {
        if (e.nodeName == "appendix")
          return formatIndex(childNumber(e), "A") + ".";
        else
          return sectionNum(e.selectSingleNode("ancestor(section|appendix)")) +
               formatIndex(childNumber(e), "1") + ".";
      }
      else
      {
        return "";
      }
    }
    function getClass(node) {
        var date = node.getAttribute("updated");
        if (date != null && date != "")
        {
          var today = new Date();
          var updated = new Date(date);
          updated.setDate(updated.getDate() + 7);
          if (today <= updated)
            return "specnew";
          updated.setDate(updated.getDate() + 60);
          if (today <= updated)
            return "specrecent";     
        }
        return "";
    }
    function getUpdated(node) {
        var updated = getClass(node);
        if (updated == "specnew")
        {
            return "background-color:yellow;";
        }
        else if (updated == "specrecent")
        {
            return "background-color:#FFFFD0;";
        }
        return "";
    }
    function LinkProto(proto) 
    {
        var updated = getClass(proto);
        if (updated != "")
        {
            updated = " class=" + updated;
        }
        var name = proto.selectSingleNode("name");
        if (name == null) name = proto.selectSingleNode("override");
        if (name != null) name = name.text;
        var code = proto.selectSingleNode("proto").text;
        var anchor = "<A " + updated + " HREF='#" + name + "'>";
        var y = code.indexOf("(");
        if (y < 0) y = code.indexOf("[");
        if (y < 0) y = code.indexOf("{");
        if (y > 0)
        {
            y--;
            while (y > 0 && code.charAt(y) == ' ')
                y--;
            var x = code.lastIndexOf(" ",y);
            if (x < 0) x = 0;
            return code.substr(0,x+1) + anchor + code.substr(x+1,y-x) + "</A>" + code.substr(y+1);
        }
        return anchor + code + "</A>";
    }
    function LinkClass(proto)
    {
        if (proto.selectSingleNode("override")) return "toc-pre-override";
        else return "toc-pre";
    }    
  ]]></xsl:script>

  <xsl:template match="element() | text()"><xsl:value-of/></xsl:template>

  <xsl:template match="attribute()"><xsl:copy><xsl:value-of/></xsl:copy></xsl:template>

<!--
  <xsl:template match="element()"><span style="background-color:red">????????????????</span></xsl:template>
    <xsl:apply-templates>
      <xsl:template><xsl:copy><xsl:apply-templates select="@*"/><xsl:apply-templates/></xsl:copy></xsl:template>
      <xsl:template match="cdata()"><xsl:value-of/></xsl:template>
    </xsl:apply-templates>
-->

  <xsl:template match="/">
    <HTML>
      <HEAD>
        <TITLE><xsl:value-of select="spec/head/title"/></TITLE>
        <LINK rel="stylesheet" type="text/css" href="xmlweb.css"/>
        <STYLE>
          .interface {margin:1em; border:2px solid rgb(52,128,184);}
          .interface-header {color:white; background-color:rgb(52,128,184); font-size:large; }
          .interface-override { font-size:medium; font-style:italic; padding-left:8px; border-top:2px solid rgb(52,128,184);color:gray }
          .interface-name { font-size:medium; padding-left:8px; border-top:2px solid rgb(52,128,184); }
          .interface-divider { font-size:medium; padding-left:8px; background-color:rgb(241,146,107); border-top:2px solid rgb(52,128,184); }
          .interface-pre { font-size:x-small;  background-color:#d0d0ff; margin:0px; }
          .interface-td{ background-color:white; font-size:x-small; border:none; margin:2px; padding:2px; text-align:left; }
          .light-name { color:#d0d0ff;}
          .toc-pre { font-size:x-small; background-color:white; margin:0px; }
          .toc-pre-override { font-size:x-small; background-color:white; color:gray; margin:0px; }
          .comment { color:rgb(40,108,154); } 
          .specnew { background-color:yellow; }
          .specrecent { background-color:#FFFFD0 }
          .typo { font-weight:bold; color:red; }
          .suggestions { background-color:#FFF0E0; border:2; border-color:#202080; }
        </STYLE>
      </HEAD>
      <BODY STYLE="font-size:x-small" onkeydown="javascript:if (event.keyCode == 118) SpellCheck()">
        <DIV id="pageBody"><xsl:apply-templates select="spec"/></DIV>
       
        <DIV id="editor" STYLE="display:none; top=0; right=-200; font-size:xx-small">
          <H1>XMLWEB Page Editor</H1>
          <TEXTAREA ID="XMLSource" STYLE="width:98%;height:500px"></TEXTAREA>
          <A class="editor-button" href="#" onclick="if (Update()) HideEditor();return false;"
            title="Preview your changes without saving them. Page remains checked out. Select 'edit this page' again to continue">preview</A>
          <A class="editor-button" href="#" onclick="window.location.reload();"
            title="Cancel editing and restore saved version.  Check out is canceled.">cancel</A>
          <A class="editor-button" href="#" onclick="Save(path);return false;"
            title="Save changes back to the server.  Page remains checked out until leaving the page.">save</A>
          <A class="editor-button" href="#" onclick="SaveAs(path);return false;"
            title="Save as a new page to an existing directory on this website.">save as...</A>
          <A class="editor-button" href="/global/help.xml"
            title="Help on using this page editor." target="_help">help</A>
          <DIV title="current status">Status: <SPAN ID="messages"></SPAN></DIV>
        </DIV>

      </BODY>
    </HTML>
  </xsl:template>

  <xsl:template match="spec">
    <xsl:for-each select="head">
      <DIV class="footer" STYLE="color:white; font-size:xx-small; top:45px">
        <xsl:apply-templates select="history/creation[date]"/>
        <xsl:for-each select="history[change/date]">
          <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Last Updated:</DIV>
          <DIV STYLE="margin-left:1em"><xsl:value-of select="change[end()]/date"/></DIV>
        </xsl:for-each>
        <xsl:if test="history/change">
          <DIV STYLE="margin-top:1em"><A href="#_revisions">Revision history</A></DIV>
        </xsl:if>
        <xsl:if test="history/change[end()]/@previous">
          <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Previous version:</DIV>
          <DIV STYLE="margin-left:1em"><A><xsl:attribute name="href"><xsl:value-of select="history/change[end()]/@previous"/></xsl:attribute><xsl:value-of select="history/change[end()]/@previous"/></A></DIV>
        </xsl:if>
        <xsl:apply-templates select="status"/>
        <xsl:apply-templates select="authors"/>
        <xsl:apply-templates select="contributors"/>
        <DIV STYLE="margin-top:1em"><A href="/XML/default.xml">Back to top page</A></DIV>
      </DIV>
    </xsl:for-each>

    <DIV id="content">
      <xsl:for-each select="head">
        <H1><xsl:value-of select="title"/></H1>

        <xsl:for-each select="abstract">
          <B><A name="Abstract">Abstract:</A></B>
          <P>
            <xsl:apply-templates/>
          </P>
        </xsl:for-each>
      </xsl:for-each>
        
      <H3><A NAME="top">Contents</A></H3>
      
      <xsl:apply-templates select="body/section">
        <xsl:template match="section[title]">
          <DIV STYLE="margin-left:1em"><xsl:eval>sectionNum(this)</xsl:eval>
            <A><xsl:attribute name="HREF">#<xsl:value-of select="title"/></xsl:attribute><xsl:value-of select="title"/></A>
            <xsl:apply-templates select="section"/>
          </DIV>
        </xsl:template>
      </xsl:apply-templates>

      <xsl:apply-templates select="appendices/appendix[title]">
        <xsl:template match="section[title]">
          <DIV STYLE="margin-left:1em"><xsl:eval>sectionNum(this)</xsl:eval>
            <A><xsl:attribute name="HREF">#<xsl:value-of select="title"/></xsl:attribute><xsl:value-of select="title"/></A>
            <xsl:apply-templates select="section"/>
          </DIV>
        </xsl:template>
        <xsl:template match="appendix[title]">
          <DIV STYLE="margin-left:1em">Appendix <xsl:eval>sectionNum(this)</xsl:eval>
            <A><xsl:attribute name="HREF">#<xsl:value-of select="title"/></xsl:attribute><xsl:value-of select="title"/></A>
            <xsl:apply-templates select="section"/>
          </DIV>
        </xsl:template>
      </xsl:apply-templates>

        <xsl:if test="head/history/change">
          <DIV STYLE="margin-left:1em">R.
            <A HREF="#_revisions">Revision history</A></DIV>
        </xsl:if>        

      <xsl:if test="//interface">
        <H3>Interfaces</H3>
        <xsl:apply-templates select="//interface">
            <xsl:template match="interface[name]">
              <DIV STYLE="margin-left:1em">
                interface <A><xsl:attribute name="HREF">#<xsl:value-of select="name"/></xsl:attribute><xsl:value-of select="name"/></A>
                <br/>{<BLOCKQUOTE style="margin:0,20">
                <xsl:apply-templates select="method[proto]|property[proto]|function[proto]|event[proto]|divider">
                   <xsl:template match="divider">
                    <PRE >// <xsl:value-of/></PRE>
                   </xsl:template>
                  <xsl:template match="method[proto]|property[proto]|function[proto]|event[proto]">
                    <PRE><xsl:attribute name="class"><xsl:eval>LinkClass(this)</xsl:eval></xsl:attribute><xsl:eval no-entities="yes">LinkProto(this)</xsl:eval></PRE>
                  </xsl:template>
                </xsl:apply-templates>
                </BLOCKQUOTE>};
              </DIV>
            </xsl:template>
        </xsl:apply-templates>
      </xsl:if>
        
      <xsl:apply-templates select="body"/>
      <xsl:apply-templates select="appendices"/>
      <DIV STYLE="font-size:xx-small; font-style:italic; text-align:right"><A HREF="#top">Back to contents</A></DIV>

      <xsl:for-each select="head/history[change/date]">
        <H2><A name="_revisions">Revisions</A></H2>
        <TABLE>
          <TR><TH>Date:</TH><TH>Change:</TH></TR>
          <xsl:for-each select="change[date]">
            <TR>
              <TD VALIGN="top"><xsl:value-of select="date"/></TD>
              <TD><xsl:for-each select="extent"><xsl:apply-templates/><BR/> </xsl:for-each></TD>
            </TR>
          </xsl:for-each>
        </TABLE>
      </xsl:for-each>
    </DIV>
  </xsl:template>

  <!-- Header templates -->
  <!-- Construct header from structured data within the <head> element -->
  <xsl:template match="creation[date]">
    <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Created:</DIV>
    <DIV STYLE="margin-left:1em"><xsl:value-of select="date"/></DIV>
  </xsl:template>

  <!-- Author templates -->
  <xsl:template match="authors">
    <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Author<xsl:if match="authors[author[1]]">s</xsl:if>:</DIV>
    <xsl:for-each select="author">
      <DIV STYLE="margin-left:1em"><A><xsl:attribute name="HREF">mailto:<xsl:value-of select="email"/></xsl:attribute>
        <xsl:value-of select="name"/></A><xsl:if match="author[company]">, <xsl:value-of select="company"/></xsl:if>
      </DIV>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="status">
    <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Status:</DIV>
    <DIV STYLE="margin-left:1em; font-weight:normal">
      <xsl:apply-templates/>
    </DIV>
  </xsl:template>

  <!-- Contributor templates -->
  <xsl:template match="contributors">
    <DIV STYLE="font-weight:bold; margin-top:1em; color:rgb(200,200,255);">Contributor<xsl:if match="contributors[contributor[1]]">s</xsl:if>:</DIV>
    <xsl:for-each select="contributor">
      <DIV STYLE="margin-left:1em"><A><xsl:attribute name="HREF">mailto:<xsl:value-of select="email"/></xsl:attribute>
        <xsl:value-of select="name"/></A><xsl:if match="contributor[company]">, <xsl:value-of select="company"/></xsl:if>
      </DIV>
    </xsl:for-each>
  </xsl:template>

  <!-- Body element templates -->
  <xsl:template match="body | appendices"><xsl:apply-templates/></xsl:template>
    
  <xsl:template match="section | appendix">
    <xsl:apply-templates select="node()[nodeName() != 'feature' and nodeName() != 'section' and nodeName() != 'appendix']"/>
    <xsl:if test="feature">
      <TABLE width="100%">
        <TR>
          <TH STYLE="background-color:#AAAACC; width:75px">Priority</TH>
          <TH STYLE="background-color:#AAAACC; width:150px">Name</TH>
          <TH STYLE="background-color:#AAAACC">Notes</TH>
          <TH STYLE="background-color:#AAAACC; width:75px">Milestone</TH>
        </TR>
        <xsl:for-each select="feature">
          <TR>
            <TD STYLE="background-color:#CCCCEE"><xsl:value-of select="priority"/></TD>
            <TD STYLE="background-color:#CCCCEE"><xsl:value-of select="name"/></TD>
            <TD STYLE="background-color:#CCCCEE"><xsl:apply-templates select="notes"/></TD>
            <TD STYLE="background-color:#CCCCEE"><xsl:value-of select="milestone"/></TD>
          </TR>
        </xsl:for-each>
      </TABLE>
    </xsl:if>
    <xsl:apply-templates select="section | appendix"/>
    <xsl:if test="context()[not(end())]">
      <div STYLE="font-size:xx-small; font-style:italic; text-align:right"><A HREF="#top">Back to contents</A></div>
    </xsl:if>
  </xsl:template>

  <xsl:template match="notes">
      <xsl:apply-templates/>
  </xsl:template>  

  <!-- section title and appendix title templates -->
  <xsl:template match="section/title">
    <H2><xsl:eval>sectionNum(this.parentNode)</xsl:eval>
    <A><xsl:attribute name="name"><xsl:value-of/></xsl:attribute><xsl:value-of/></A></H2>
  </xsl:template>

  <xsl:template match="section/section/title">
    <H3><xsl:eval>sectionNum(this.parentNode)</xsl:eval>
    <A><xsl:attribute name="name"><xsl:value-of/></xsl:attribute><xsl:value-of/></A></H3>
  </xsl:template>

  <xsl:template match="section/section/section/title">
    <H4><xsl:eval>sectionNum(this.parentNode)</xsl:eval>
    <A><xsl:attribute name="name"><xsl:value-of/></xsl:attribute><xsl:value-of/></A></H4>
  </xsl:template>

  <xsl:template match="section/section/section/section/title">
    <H5><xsl:eval>sectionNum(this.parentNode)</xsl:eval>
    <A><xsl:attribute name="name"><xsl:value-of/></xsl:attribute><xsl:value-of/></A></H5>
  </xsl:template>

  <xsl:template match="appendix/title">
    <H2>Appendix <xsl:eval>sectionNum(this.parentNode)</xsl:eval>
    <A><xsl:attribute name="name"><xsl:value-of/></xsl:attribute><xsl:value-of/></A></H2>
  </xsl:template>  

  <!-- Code example templates -->

  <xsl:template match="example">
    <BLOCKQUOTE><xsl:if match="example[@color]"><xsl:attribute name="STYLE">background-color:<xsl:value-of/></xsl:attribute></xsl:if><FONT SIZE="3">
      <PRE><xsl:apply-templates>
	<xsl:template match="textnode()"><xsl:value-of/></xsl:template>
        <xsl:template match="comment"><span class="comment"><xsl:value-of/></span></xsl:template>
	</xsl:apply-templates></PRE>
    </FONT></BLOCKQUOTE>
  </xsl:template>

  <xsl:template match="appendix/example">
    <FONT SIZE="3">
      <PRE><xsl:apply-templates/></PRE>
    </FONT>
  </xsl:template>

  <xsl:template match="example/description"/>

  <xsl:template match="sample">
    <xsl:apply-templates>
      <xsl:template><xsl:copy><xsl:apply-templates select="@*"/><xsl:apply-templates/></xsl:copy></xsl:template>
      <xsl:template match="cdata()"><xsl:value-of/></xsl:template>
    </xsl:apply-templates>
  </xsl:template>

  <!-- List templates -->
  <xsl:template match="bullet-list">
    <UL>
      <xsl:for-each select="item">
        <LI><xsl:apply-templates/></LI>
      </xsl:for-each>
    </UL>
  </xsl:template>

  <xsl:template match="ul">
    <UL>
      <xsl:for-each select="li">
        <LI><xsl:apply-templates/></LI>
      </xsl:for-each>
    </UL>
  </xsl:template>

  <xsl:template match="number-list">
    <OL>
      <xsl:for-each select="item">
        <LI><xsl:apply-templates/></LI>
      </xsl:for-each>
    </OL>
  </xsl:template>

  <xsl:template match="ol">
    <OL>
      <xsl:for-each select="li">
        <LI><xsl:apply-templates/></LI>
      </xsl:for-each>
    </OL>
  </xsl:template>

  <xsl:template match="definition-list">
    <TABLE>
      <xsl:apply-templates select="term">
      <xsl:template match="term">
        <TR>
          <TD><xsl:apply-templates/></TD>
          <TD><xsl:eval>selectSingleNode("../definition["+(childNumber(this)-1)+"]").text</xsl:eval></TD>
        </TR>
      </xsl:template>
      </xsl:apply-templates>
    </TABLE>
  </xsl:template>

  <xsl:template match="dl">  
    <TABLE>
      <xsl:apply-templates select="dt">
      <xsl:template match="dt">
        <TR>
          <TD><xsl:apply-templates/></TD>
          <TD><xsl:eval>selectSingleNode("../dd["+(childNumber(this)-1)+"]").text</xsl:eval></TD>
        </TR>
      </xsl:template>
      </xsl:apply-templates>
    </TABLE>
  </xsl:template>

  <xsl:template match="history-list">
    <DL>
      <xsl:apply-templates />
    </DL>
  </xsl:template>

  <xsl:template match="step-list">
    <DL>
      <xsl:for-each select="item">
        <DT><BR/><B>Step <eval>formatNumber(childNumber(this), "1")</eval>:
        <xsl:value-of select="step-name"/></B></DT>
        <DD><xsl:value-of select="step-text"/></DD>
      </xsl:for-each>
    </DL>
  </xsl:template>

  <xsl:template match="history-list/item">
    <DD>
      <xsl:apply-templates/>
    </DD>
  </xsl:template>
    
  <xsl:template match="history-list/item">
    <DT><B><eval>getAttribute("date")</eval></B></DT>
    <DD>
      <xsl:apply-templates/>
    </DD>
  </xsl:template>

  <!-- FAQ -->

  <xsl:template match="faq">
    <BLOCKQUOTE>
      <xsl:apply-templates/>
    </BLOCKQUOTE>
  </xsl:template>

  <xsl:template match="question">
      <DT>
    <B>Question: </B>
    </DT>
    <DD>
      <I><xsl:apply-templates/></I>
    </DD>
  </xsl:template>

  <xsl:template match="answer">
    <DT>
      <B>Answer: </B>
    </DT>
    <DD>
      <xsl:apply-templates/>
    </DD>
  </xsl:template>

  <!-- figures and graphics -->

  <xsl:template match="figure">
    <CENTER>
      <xsl:apply-templates select="graphic"/>
      <BR/>
      <xsl:apply-templates select=".//name"/>
    </CENTER>
    <xsl:apply-templates select="map"/>
  </xsl:template>

  <xsl:template match="graphic">
    <IMG>
      <xsl:attribute name="SRC"><xsl:value-of/></xsl:attribute>
      <xsl:attribute name="usemap"><xsl:value-of select="@usemap"/></xsl:attribute>
      <xsl:attribute name="border"><xsl:value-of select="@border"/></xsl:attribute>
    </IMG>
  </xsl:template>

  <xsl:template match="figure/name">
      <I><xsl:apply-templates/></I>
  </xsl:template>  

  <xsl:template match="figure/map">
    <xsl:copy>        
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>  

  <xsl:template match="figure/map/@*">
    <xsl:copy><xsl:value-of/></xsl:copy>
  </xsl:template>  

  <xsl:template match="figure/map/area">
    <xsl:copy>        
      <xsl:apply-templates select="@*"/>
    </xsl:copy>
  </xsl:template>  

  <xsl:template match="figure/map/area/@*">
    <xsl:copy><xsl:value-of/></xsl:copy>
  </xsl:template>  

  <!-- method templates -->
  <xsl:template match="class">
    <DIV class="class"><DIV class="interface-header"><SPAN style="color:black; font-weight:bold"><xsl:value-of select="name"/></SPAN> class </DIV>
    <xsl:apply-templates select="method|property|function|event|divider"/>
    </DIV>
  </xsl:template>

  <xsl:template match="interface">
    <DIV class="interface"><DIV class="interface-header"><SPAN style="color:black; font-weight:bold">
        <A><xsl:attribute name="NAME"><xsl:value-of select="name"/></xsl:attribute><xsl:value-of select="name"/></A>
    </SPAN> interface </DIV>
    <xsl:apply-templates select="method|property|function|event|divider"/>
    </DIV>
  </xsl:template>

  <xsl:template match="divider">
    <DIV class="interface-divider"><xsl:value-of/></DIV>
  </xsl:template>

  <xsl:template match="method|property|function|event">
     <xsl:if test="override">
        <DIV class="interface-override">
            <xsl:attribute name="style"><xsl:eval>getUpdated(this)</xsl:eval></xsl:attribute>
            <A><xsl:attribute name="NAME"><xsl:value-of select="override"/></xsl:attribute><xsl:value-of select="override"/></A>
          <SPAN class="light-name" ><xsl:node-name/> override</SPAN>
        </DIV>
     </xsl:if>
     <xsl:if test="name">
        <DIV class="interface-name">
            <xsl:attribute name="style"><xsl:eval>getUpdated(this)</xsl:eval></xsl:attribute>
          <A><xsl:attribute name="NAME"><xsl:value-of select="name"/></xsl:attribute><xsl:value-of select="name"/></A>
          <SPAN class="light-name" ><xsl:node-name/></SPAN>
        </DIV>
    </xsl:if>
    <DL STYLE="margin-left:22px">
      <DT><B>Prototype:</B></DT>
      <DD><pre class="interface-pre"><xsl:value-of select="proto"/></pre></DD>
      <DT><B>Description:</B></DT>
      <DD><xsl:apply-templates select="purpose"/></DD>
      <xsl:for-each select="args">
        <DT><B>Arguments:</B></DT>
        <DD><TABLE class="interface-table">
          <TBODY>
            <xsl:for-each select="arg">
              <TR>
                <TD VALIGN="top" class="interface-td">
                  <I><xsl:value-of select="name"/></I>
                </TD>
                <TD width="10px" class="interface-td"></TD>
                <TD VALIGN="top" class="interface-td">
                  <xsl:apply-templates select="usage"/>
                </TD>
              </TR>
            </xsl:for-each>
          </TBODY>
        </TABLE></DD>
      </xsl:for-each>
      
      <xsl:for-each select="returns">
        <DT><B>Returns:</B></DT>
        <DD><xsl:apply-templates/></DD>
      </xsl:for-each>

      <xsl:for-each select="example">
        <DT><B>Examples:</B></DT>
        <DD><PRE><xsl:value-of/></PRE></DD>
      </xsl:for-each>
    </DL>
  </xsl:template>
    
  <xsl:template match="usage">
      <xsl:apply-templates/>
  </xsl:template>  


  <!-- link templates -->

  <xsl:template match="external-link">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of/></xsl:attribute>
      <xsl:if match="external-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="external-link[@to]">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of select="@to"/></xsl:attribute>
      <xsl:if match="external-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="a">
    <A><xsl:copy><xsl:apply-templates select="@*"/><xsl:apply-templates/></xsl:copy></A>
  </xsl:template>

  <xsl:template match="img">
    <xsl:copy><xsl:apply-templates select="@*"/><xsl:apply-templates/></xsl:copy>
  </xsl:template>

  <xsl:template match="internal-link">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of/></xsl:attribute>
      <xsl:if match="internal-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="internal-link[@to]">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of select="@to"/></xsl:attribute>
      <xsl:if match="internal-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="nearby-link">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of/></xsl:attribute>
      <xsl:if match="nearby-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="nearby-link[@to]">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of select="@to"/></xsl:attribute>
      <xsl:if match="nearby-link[@replace='yes']">
        <xsl:attribute name="TARGET">_top</xsl:attribute>
      </xsl:if>
      <xsl:for-each select="@target">
        <xsl:copy><xsl:value-of/></xsl:copy>
      </xsl:for-each>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="local-link">
    <A>
      <xsl:attribute name="HREF">#<xsl:value-of/></xsl:attribute>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="local-link[@to]">
    <A>
      <xsl:attribute name="HREF">#<xsl:value-of select="@to"/></xsl:attribute>
      <xsl:value-of/></A></xsl:template>

  <xsl:template match="download">
    <A>
      <xsl:attribute name="HREF"><xsl:value-of select="from"/></xsl:attribute>
      <IMG SRC="/sitebuilder/graphics/icodownl.gif" WIDTH="16" HEIGHT="20" BORDER="0" ALT="Download"/>
      <xsl:apply-templates/></A></xsl:template>

  <!-- Style rules -->
  <xsl:template match="emph|key-term"><I><xsl:apply-templates/></I></xsl:template>

  <xsl:template match="i"><I><xsl:apply-templates/></I></xsl:template>

  <xsl:template match="b"><B><xsl:apply-templates/></B></xsl:template>

  <xsl:template match="strong"><B><xsl:apply-templates/></B></xsl:template>

  <xsl:template match="code"><FONT face="Courier"><xsl:apply-templates/></FONT></xsl:template>

  <xsl:template match="function-name|property-name|method-name|attribute-name"><B><xsl:apply-templates/></B></xsl:template>

  <xsl:template match="tag"><B>&lt;<xsl:apply-templates/>&gt;</B></xsl:template>

  <xsl:template match="filename"><xsl:apply-templates/></xsl:template>
  
  
  <xsl:template match="link">
		<A>
				<xsl:attribute name="href"><xsl:value-of select="location"/></xsl:attribute>
				<xsl:attribute name="title">(<xsl:if test="@updated">last updated on <xsl:value-of select="@updated"/></xsl:if><xsl:if test="@by"> by <xsl:value-of select="@by"/></xsl:if>)</xsl:attribute>
				<xsl:choose>
					<xsl:when test="title"><xsl:value-of select="title"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="location"/></xsl:otherwise>
				</xsl:choose>
			</A>
		<DIV class="link-description"><xsl:apply-templates select="description"/></DIV>
	</xsl:template>
  
  <!-- document elements templates -->

  <xsl:template match="p">    
    <P><xsl:attribute name="class"><xsl:eval>getClass(this)</xsl:eval></xsl:attribute><xsl:apply-templates/></P>
  </xsl:template>

  <xsl:template match="purpose">
    <P><xsl:apply-templates/></P>
  </xsl:template>

  <xsl:template match="issue">
    <P><FONT color="red"><SMALL><B>Open issue: <xsl:apply-templates/></B></SMALL></FONT></P>
  </xsl:template>

  <xsl:template match="note">
    <P><SMALL><I>Note: <xsl:apply-templates/></I></SMALL></P>
  </xsl:template>

  <xsl:template match="hint">
    <P><SMALL><I>Hint: <xsl:apply-templates/></I></SMALL></P>
  </xsl:template>

</xsl:stylesheet>
